@@ -13,7 +13,9 @@ use async_trait::async_trait;
 
 use tokio::net::UnixDatagram as TokioUnixDatagram;
 
-use super::worker::{AuthMessageType, ParentToSessionChild, SessionChildToParent, TerminalMode};
+use super::worker::{
+    AuthMessageType, ParentToSessionChild, SessionChildToParent, SessionClass, TerminalMode,
+};
 use crate::error::Error;
 
 #[async_trait]
@@ -27,7 +29,7 @@ trait AsyncSend {
 }
 
 #[async_trait]
-impl AsyncSend for ParentToSessionChild {
+impl<'a> AsyncSend for ParentToSessionChild<'a> {
     async fn send(&self, sock: &mut TokioUnixDatagram) -> Result<(), Error> {
         let out =
             serde_json::to_vec(self).map_err(|e| format!("unable to serialize message: {}", e))?;
@@ -97,8 +99,7 @@ impl Session {
             UnixDatagram::pair().map_err(|e| format!("could not create pipe: {}", e))?;
 
         let raw_child = childfd.as_raw_fd();
-        let mut cur_flags =
-            unsafe { FdFlag::from_bits_unchecked(fcntl(raw_child, FcntlArg::F_GETFD)?) };
+        let mut cur_flags = FdFlag::from_bits_retain(fcntl(raw_child, FcntlArg::F_GETFD)?);
         cur_flags.remove(FdFlag::FD_CLOEXEC);
         fcntl(raw_child, FcntlArg::F_SETFD(cur_flags))?;
 
@@ -133,19 +134,21 @@ impl Session {
     pub async fn initiate(
         &mut self,
         service: &str,
-        class: &str,
+        class: SessionClass,
         user: &str,
         authenticate: bool,
         term_mode: &TerminalMode,
         source_profile: bool,
+        listener_path: &str,
     ) -> Result<(), Error> {
         let msg = ParentToSessionChild::InitiateLogin {
-            service: service.to_string(),
-            class: class.to_string(),
-            user: user.to_string(),
+            service: service,
+            class: class,
+            user: user,
             authenticate,
             tty: term_mode.clone(),
             source_profile,
+            listener_path: listener_path,
         };
         msg.send(&mut self.sock).await?;
         Ok(())
